@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /*
@@ -26,6 +25,7 @@
 #include <fvwmlib.h>
 #include "FRenderInit.h"
 #include "XError.h"
+#undef XSetErrorHandler
 
 #define USE_GET_ERROR_TEXT 1
 #ifndef USE_GET_ERROR_TEXT
@@ -34,6 +34,20 @@ static char *error_name(unsigned char code);
 static char *request_name(unsigned char code);
 static char unknown[32];
 
+void do_coredump(void)
+{
+	fprintf(stderr, " Leaving a core dump now\n");
+	{
+		char *nullp;
+
+		nullp = NULL;
+
+		*nullp = 99;
+	}
+	/* exit if this fails */
+	exit(99);
+}
+
 #define USE_GET_ERROR_TEXT 1
 void PrintXErrorAndCoredump(Display *dpy, XErrorEvent *error, char *MyName)
 {
@@ -76,10 +90,7 @@ void PrintXErrorAndCoredump(Display *dpy, XErrorEvent *error, char *MyName)
 		error->resourceid);
 
 	/* leave a coredump */
-	fprintf(stderr, " Leaving a core dump now\n");
-	abort();
-	/* exit if this fails */
-	exit(99);
+	do_coredump();
 }
 
 #ifndef USE_GET_ERROR_TEXT
@@ -254,3 +265,43 @@ static char *request_name(unsigned char code)
 	}
 	return code_names[code - 1];
 }
+
+/* -------------------------- error handler stack -------------------------- */
+
+static ferror_handler_t old_handler = NULL;
+
+void ferror_set_temp_error_handler(ferror_handler_t new_handler)
+{
+	if (old_handler != NULL)
+	{
+		do_coredump();
+	}
+	old_handler = XSetErrorHandler(old_handler);
+
+	return;
+}
+
+void ferror_reset_temp_error_handler(void)
+{
+	if (old_handler == NULL)
+	{
+		do_coredump();
+	}
+	XSetErrorHandler(old_handler);
+	old_handler = NULL;
+
+	return;
+}
+
+int ferror_call_next_error_handler(Display *dpy, XErrorEvent *error)
+{
+	int rc;
+
+	if (old_handler == NULL)
+	{
+		do_coredump();
+	}
+	rc = old_handler(dpy, error);
+
+	return rc;
+}
